﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;

namespace APLNext.SCS
{
    class Program
    {
        static APLNext.Supervisor S = new APLNext.Supervisor();
        static StringBuilder ProcessCompleteResult = new StringBuilder();
        //^ProcessCompleteResult: Accumulates the result of each kernel function execution
        static Int32 NumberOfProcessComplete = 0;
        static void Main(string[] args)
        {
            string testPath = @"C:\APLNext\APLNextSupervisor\";
            //^ APLNext Supervisor is assumed to be installed to c:\APLNext\ folder
            string wsName = "SUPERVISORTEST.w3";
            //^ Test workspace is assumed to exist in testPath folder
 
            string SLog = Path.Combine(testPath, "Spvr.log");
            File.Delete(SLog);
            S.SetLogFileName(SLog);

            StringBuilder sbConfig = new StringBuilder();
            sbConfig.Append(@"<?xml version='1.0'?>");
            sbConfig.Append(@"<config><workspaces><workspace id='SUPERVISORTEST'>");
            sbConfig.Append(@"<minpool>1</minpool>");
            string maxPool = (S.NProcessors[1] - 2).ToString();
            sbConfig.Append(@"<maxpool>"+maxPool+"</maxpool>");
            //^Set maximum #APL+Win ActiveX engine instances based on #processors on target workstation
            sbConfig.Append(@"<timeout>30000</timeout>");
            sbConfig.Append(@"<debug>1</debug>");
            //^For testing purposes ignore timeout
            // For production purposes set debug parameter value to 0 to consider timeout
            sbConfig.Append(@"<visible>1</visible>");
            sbConfig.Append(@"<wslocation>" + Path.Combine(testPath, wsName) + "</wslocation>");
            sbConfig.Append(@"<wssize>1600000</wssize>");
            sbConfig.Append(@"<evlevel>2</evlevel>");
            sbConfig.Append(@"<busyid />");
            sbConfig.Append(@"<user_state>started</user_state>");
            sbConfig.Append(@"</workspace></workspaces></config>");
            S.OpenConfigFromText(sbConfig.ToString());
            bool beginCallResult;
            try
            {
                S.Start();
                S.ProcessCompleteCallback += new ProcessCompleteHandler(S_ProcessCompleteCallback);
                //^ Subscribe to the ProcessCompleteCallback event with the S_ProcessCompleteCallBack event handler method
                S.ProcessProgressCallback += S_ProcessProgressCallback;
                //^ Subscribe to the ProcessProgressCallback event with the S_ProcessProgressCallback event handler method
                for (Int32 I = 1; I <= 5; I++)
                {
                    beginCallResult = false;
                    beginCallResult = S.BeginCall(I, "SUPERVISORTEST", "KernelDyadic", "rarg=" + I.ToString(), "larg=" + I.ToString());
                    //^ Request the asynchronous execution of the 'KernelDyadic' function
                    //  in the 'SUPERVISORTEST' workspace
                    //  with right argument value: "rarg=" + I.ToString()
                    if (!beginCallResult)
                    {
                        Console.WriteLine("BeginCall failed: processId: " + I.ToString());
                    }
                    else
                    {
                        Console.WriteLine("BeginCall OK: id: " + I.ToString());
                    }
                }
               S.ProcessAllDoneCallback += new ProcessAllDoneHandler(S_ProcessAllDoneCallback);
                //^ Subscribe to the ProcessAllDoneCallback event with the S_ProcessAllDoneCallback event handler method
                //  after all tasks have been queued using the 'BeginCall' method
                System.Diagnostics.Process.GetCurrentProcess().WaitForExit();
                //^Wait until the ProcessAllDoneCallback event has fired
            }
            catch (Exception e)
            {
                Console.WriteLine("Error in Controlling Application: " + e.Message);
                S.Stop();
                Console.ReadLine();
            }
        }

        static void S_ProcessProgressCallback(int id, object code, object data)
        {
            //id  : processId provided by 'controlling app' as a 'BeginCall' argument
            //code: Object containing 'code' information provided by the 'kernel' function when
            //      it used the APL+Win 'Notify' event
            //data: Object containing 'data' information provided by the 'kernel' function when
            //      it used the APL+Win 'Notify' method
            // In order to process 'code' and 'data', the C# program must 'know a priori' the
            // structure of the data provided by APL+Win.  This is because the .Net datatype of
            // 'code' and 'data' is object
            // The data structures of 'code' and 'data' are independent
            //
            // In the SUPERVISORTEST.w3 APL+Win workspace associated with the APLNext Supervisor v1.9.2.0
            // 'code' is object[2] with object[2][0] string and object[2][1] a matrix: Int32[,]
            // 'data' is object[2] with object[2][0] string and object[2][1] a matrix: Int32[,]
            // Review the 'KernelDyadic' APL+Win for details of the application-specific data being provided
            // to the 'controlling application' via the APL+Win 'Notify' method and the
            // APLNext Supervisor'ProcessProgressCallback' event
            Console.WriteLine("In S_ProcessProgressCallback");
            Console.WriteLine("Info from 'kernel' function: ProcessId: " + id.ToString());
            object[] codeFromNotify = (object[])code;
            Console.WriteLine("1st element of 'code' argument to Notify [string]: " + (string)(codeFromNotify[0]));
            Console.WriteLine("2nd element of 'code' argument to Notify [matrix]: ");
            DisplayMatrix((Int32[,])(codeFromNotify[1]));
            object[] dataFromNotify = (object[])code;
            Console.WriteLine("1st element of 'data' argument to Notify [string]: " + (string)(dataFromNotify[0]));
            Console.WriteLine("2nd element of 'data' argument to Notify [matrix]: ");
            DisplayMatrix((Int32[,])(dataFromNotify[1]));
        }
        private static void DisplayMatrix(Int32[,] mat)
        {
            for (Int32 r = 0; r < mat.GetLength(0); r++)
            {
                for (Int32 c = 0; c < mat.GetLength(1); c++)
                {
                    Console.WriteLine("Row #" + r.ToString() + " Col #" + c.ToString() + " Value: " + mat[r, c].ToString());
                }
            }
        }
        static void S_ProcessCompleteCallback(int id, int errcode, object resultData)
        {
            //This ProcessCompleteCallback event handler method accumulates
            //the 'kernel' function results in the ProcessCompleteResult StringBuilder object
            NumberOfProcessComplete++; 
            StringBuilder sb = new StringBuilder();
            sb.Append("ProcessComplete: id: " + id.ToString() + " errcode: " + errcode.ToString());
            if (0 != errcode)
            {
                sb.Append(" errDesc(in resultData):");
                sb.AppendLine(resultData.ToString());
            }
            else
            {
                if (resultData == null)
                {
                    sb.Append(" resultData: null");
                }
                else
                {
                    sb.Append(" resultData: " + resultData.ToString());
                }
            }
            ProcessCompleteResult.AppendLine(sb.ToString());
            Console.WriteLine(sb.ToString());
        }
        static void S_ProcessAllDoneCallback()
        {
            //This ProcessAllDoneCallback event handler method consolidates, analyzes and displays
            //the 'kernel' function results accumulated by each ProcessCompleteCallback event
            Console.WriteLine();
            Console.WriteLine("S_ProcessAllDoneCallback():");
            Console.WriteLine("ProcessCompleteResult:");
            Console.WriteLine("#ProcessCompleteCallback events fired: " + NumberOfProcessComplete.ToString());
            Console.WriteLine(ProcessCompleteResult.ToString());
            //^ Display the results of all kernel function executions
            Console.WriteLine("Click Enter to end this program");
            Console.ReadLine();
            S.Stop();
            //^ Always stop the APLNext Supervisor
            Environment.Exit(0);
            //^End this application now that the ProcessAllDoneCallback has fired
        }
    }
}
