﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.IO;

namespace APLNext.SCSsync
{
    class Program
    {
        static APLNext.Supervisor S = new APLNext.Supervisor();
        static void Main(string[] args)
        //^Main() is the 'Controlling App'
        {
            string testPath = @"C:\APLNext\APLNextSupervisor\";
            string wsName = "SUPERVISORTEST.w3";
            //^ Test workspace is assumed to exist in testPath folder

            string SLog = Path.Combine(testPath, "Spvr.log");
            File.Delete(SLog);
            S.SetLogFileName(SLog);

            StringBuilder sbConfig = new StringBuilder();
            sbConfig.Append(@"<?xml version='1.0'?>");
            sbConfig.Append(@"<config><workspaces><workspace id='SUPERVISORTEST'>");
            sbConfig.Append(@"<minpool>1</minpool>");
            string maxPool = (S.NProcessors[1] - 1).ToString();
            sbConfig.Append(@"<maxpool>" + maxPool + "</maxpool>");
            //^Set maximum #APL+Win ActiveX engine instances based on #processors on target workstation
            sbConfig.Append(@"<timeout>30000</timeout>");
            sbConfig.Append(@"<debug>1</debug>");
            //^For testing purposes ignore timeout
            // For production purposes set debug parameter value to 0 to consider timeout
            sbConfig.Append(@"<visible>1</visible>");
            sbConfig.Append(@"<wslocation>" + Path.Combine(testPath, wsName) + "</wslocation>");
            sbConfig.Append(@"<wssize>1600000</wssize>");
            sbConfig.Append(@"<evlevel>2</evlevel>");
            sbConfig.Append(@"<busyid />");
            sbConfig.Append(@"<user_state>started</user_state>");
            sbConfig.Append(@"</workspace></workspaces></config>");
            S.OpenConfigFromText(sbConfig.ToString());
            bool beginCallResult;
            try
            {
                S.StartSync();
                //^ Tasks submitted to the APLNext Supervisor queue will not commence processing
                //   until the ExecuteAPL() method is used
                for (Int32 I = 1; I <= 5; I++)
                {
                    beginCallResult = false;
                    beginCallResult = S.BeginCall(I, "SUPERVISORTEST", "KernelMonadicSync", "rarg=" + I.ToString(), null);
                    //^ Submit a task to the APLNext Supervisor processing queue
                    if (!beginCallResult)
                    {
                        Console.WriteLine("BeginCall failed: processId: " + I.ToString());
                    }
                    else
                    {
                        Console.WriteLine("BeginCall OK: id: " + I.ToString());
                    }
                }
                Console.WriteLine("Using the APLNext Supervisor 'ExecuteAPL()' method");
                Console.WriteLine("Execution of the C# 'Controlling App' will be suspended");
                Console.WriteLine(" while the tasks are processed by APL+Win ActiveX engine instances");
                Console.WriteLine(" running the kernel function.");
                S.ExecuteAPL();
                //^ The ExecuteAPL() method suspends the execution of the Controlling App method
                //   until all queued tasks have been processed
                string fnm = @"C:\APLNEXT\APLNEXTSUPERVISOR\SyncResults.cf";

                Console.WriteLine(@"Results are in the "+fnm+ " colossal component file");
                Console.WriteLine("Stopping the APLNext Supervisor instance");
                S.Stop();
                Console.WriteLine("Erase the '"+fnm+"' file");
                Console.ReadLine();
                File.Delete(fnm);
                Console.WriteLine("Click Enter to end this C# 'Controlling App' program");
                Console.ReadLine();
                //^ Always stop the APLNext Supervisor
                Environment.Exit(0);
                //^End this application now that the ProcessAllDoneCallback has fired
            }
            catch (Exception e)
            {
                Console.WriteLine("Error in Controlling Application: " + e.Message);
                S.Stop();
                Console.ReadLine();
            }
        }

    }
}
