﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;

namespace TestPauseResume
{
    class Program
    {
        static APLNext.Supervisor S = new APLNext.Supervisor();
        static StringBuilder ProcessCompleteResult = new StringBuilder();
        //^ProcessCompleteResult: Accumulates the result of each kernel function execution
       
        static void Main(string[] args)
        {
            string testPath = @"C:\APLNext\APLNextSupervisor\";
            string wsName = "SUPERVISORTEST.w3";
            string SLog = Path.Combine(testPath, "Spvr.log");
            File.Delete(SLog);
            S.SetLogFileName(SLog);

            StringBuilder sbConfig = new StringBuilder();
            sbConfig.Append(@"<?xml version='1.0'?>");
            sbConfig.Append(@"<config><workspaces><workspace id='SUPERVISORTEST'>");
            sbConfig.Append(@"<minpool>1</minpool>");
            sbConfig.Append(@"<maxpool>2</maxpool>");
            sbConfig.Append(@"<timeout>15000</timeout>");
            sbConfig.Append(@"<debug>0</debug>");
            sbConfig.Append(@"<visible>1</visible>");
            sbConfig.Append(@"<wslocation>" + Path.Combine(testPath, wsName) + "</wslocation>");
            sbConfig.Append(@"<wssize>1600000</wssize>");
            sbConfig.Append(@"<evlevel>2</evlevel>");
            sbConfig.Append(@"<busyid />");
            sbConfig.Append(@"<user_state>started</user_state>");
            sbConfig.Append(@"</workspace></workspaces></config>");
            S.OpenConfigFromText(sbConfig.ToString());

            bool beginCallResult;
            try
            {
                S.Start();
                S.ProcessCompleteCallback +=new APLNext.ProcessCompleteHandler(S_ProcessCompleteCallback);
                for (Int32 I = 1; I <= 10; I++)
                {
                    beginCallResult = false;
                    if (I == 6)
                    {
                        Console.WriteLine("Supervisor Pause()");
                        S.Pause();
                        Console.WriteLine("Supervisor QueueSize: " + S.QueueSize.ToString());
                        Console.WriteLine("Thread.Sleep(5000) - About 5 sec.");
                        System.Threading.Thread.Sleep(5000);
                        Console.WriteLine("Supervisor Resume()");
                        S.Resume();
                    }
                    beginCallResult = S.BeginCall(I, "SUPERVISORTEST", "KernelMonadic", "rarg=" + I.ToString(), null);
                    if (!beginCallResult)
                    {
                        Console.WriteLine("BeginCall failed: processId: " + I.ToString());
                    }
                    else
                    {
                        Console.WriteLine("BeginCall OK: id: " + I.ToString());
                    }
                }
                S.ProcessAllDoneCallback +=new APLNext.ProcessAllDoneHandler(S_ProcessAllDoneCallback);
                System.Diagnostics.Process.GetCurrentProcess().WaitForExit();
                //^Wait until the ProcessAllDoneCallback event has fired
            }
            catch (Exception e)
            {
                Console.WriteLine("Error in Main(): " + e.Message);
                S.Stop();
                Console.ReadLine();
            }
        }
        static void S_ProcessCompleteCallback(int id, int errcode, object resultData)
        {
            StringBuilder sb = new StringBuilder();
            sb.Append("ProcessComplete: id: " + id.ToString() + " errcode: " + errcode.ToString());
            if (0 != errcode)
            {
                sb.Append(" errDesc(in resultData):");
                sb.AppendLine(resultData.ToString());
            }
            else
            {
                if (resultData == null)
                {
                    sb.Append(" resultData: null");
                }
                else
                {
                    sb.Append(" resultData: " + resultData.ToString());
                }

            }
            Console.WriteLine(sb.ToString());
            ProcessCompleteResult.AppendLine(sb.ToString());
        }
        static void S_ProcessAllDoneCallback()
        {
            Console.WriteLine();
            Console.WriteLine("S_ProcessAllDoneCallback():");
            Console.WriteLine("ProcessCompleteResult:");
            Console.WriteLine(ProcessCompleteResult.ToString());
            //^ Display the results of all kernel function executions
            Console.WriteLine("Click Enter to end this program");
            Console.ReadLine();
            S.Stop();
            Environment.Exit(0);
            //^End this application now that the ProcessAllDoneCallback has fired
        }
    }
}