﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;

namespace TestPauseThreadsOnError
{
    class Program
    {
        static APLNext.Supervisor S = new APLNext.Supervisor();
        static StringBuilder ProcessCompleteResult = new StringBuilder();
        //^ProcessCompleteResult: Accumulates the result of each kernel function execution
        static void Main(string[] args)
        {
            string testPath = @"C:\APLNext\APLNextSupervisor\";
            string wsName = "SUPERVISORTEST.w3";
            //^ Test workspace is assumed to exist in testPath folder

            string SLog = Path.Combine(testPath, "Spvr.log");
            File.Delete(SLog);
            S.SetLogFileName(SLog);

            StringBuilder sbConfig = new StringBuilder();
            sbConfig.Append(@"<?xml version='1.0'?>");
            sbConfig.Append(@"<config><workspaces><workspace id='SUPERVISORTEST'>");
            sbConfig.Append(@"<minpool>1</minpool>");
            sbConfig.Append(@"<maxpool>2</maxpool>");
            sbConfig.Append(@"<timeout>300000</timeout>");
            sbConfig.Append(@"<debug>0</debug>");
            sbConfig.Append(@"<visible>1</visible>");
            sbConfig.Append(@"<wslocation>" + Path.Combine(testPath, wsName) + "</wslocation>");
            sbConfig.Append(@"<wssize>1600000</wssize>");
            sbConfig.Append(@"<evlevel>2</evlevel>");
            sbConfig.Append(@"<busyid />");
            sbConfig.Append(@"<user_state>started</user_state>");
            sbConfig.Append(@"</workspace></workspaces></config>");
            S.OpenConfigFromText(sbConfig.ToString());

            bool beginCallResult;
            try
            {
                S.Start();
                S.PauseThreadsOnError = true;
                //^ When an error condition is detected by the APLNext Supervisor, any uncompleted 'kernel' function processing threads will be paused

                S.ProcessCompleteCallback += new APLNext.ProcessCompleteHandler(S_ProcessCompleteCallback);
                //^ Subscribe to the ProcessCompleteCallback event with the S_ProcessCompleteCallBack event handler method
                
                for (Int32 I = 1; I <= 25; I++)
                {
                    beginCallResult = false;
                    if (I == 22)
                    {
                        Console.WriteLine("Supervisor QueueSize before error: " + S.QueueSize.ToString());
                        beginCallResult = S.BeginCall(I, "SUPERVISORTEST", "NonExistentFunction", "rarg=" + I.ToString(), null);
                        //^ Generate an error condition that the APLNext Supervisor can detect
                        Console.WriteLine("Supervisor QueueSize after error: " + S.QueueSize.ToString());
                    }
                    else
                    {
                        beginCallResult = S.BeginCall(I, "SUPERVISORTEST", "KernelMonadic", "rarg=" + I.ToString(), null);
                        //^ Request the asynchronous execution of the 'KernelMonadic' function
                        //  in the 'SUPERVISORTEST' workspace
                        //  with right argument value: "rarg=" + I.ToString()
                    }
                    if (!beginCallResult)
                    {
                        Console.WriteLine("BeginCall failed: processId: " + I.ToString());
                    }
                    else
                    {
                        Console.WriteLine("BeginCall OK: id: " + I.ToString());
                    }
                }
                S.ProcessAllDoneCallback += new APLNext.ProcessAllDoneHandler(S_ProcessAllDoneCallback);
                //^ Subscribe to the ProcessAllDoneCallback event with the S_ProcessAllDoneCallback event handler method

                Console.WriteLine("Supervisor QueueSize after all processing requests have been made: " + S.QueueSize.ToString());
                Console.WriteLine("Click Enter to Resume processing after PauseThreadsOnError");
                Console.ReadLine();
                S.PauseThreadsOnError = false;
                S.Resume();
                
                System.Diagnostics.Process.GetCurrentProcess().WaitForExit();
                //^Wait until the ProcessAllDoneCallback event has fired
            }
            catch (Exception e)
            {
                Console.WriteLine("Error in Controlling Application: " + e.Message);
                S.Stop();
                Console.ReadLine();
            }
        }
        static void S_ProcessCompleteCallback(int id, int errcode, object resultData)
        {
            //This ProcessCompleteCallback event handler method accumulates
            //the 'kernel' function results in the ProcessCompleteResult StringBuilder object
            StringBuilder sb = new StringBuilder();
            sb.Append("ProcessComplete: id: " + id.ToString() + " errcode: " + errcode.ToString());
            if (0 != errcode)
            {
                sb.Append(" errDesc(in resultData):");
                sb.AppendLine(resultData.ToString());
            }
            else
            {
                if (resultData == null)
                {
                    sb.Append(" resultData: null");
                }
                else
                {
                    sb.Append(" resultData: " + resultData.ToString());
                }

            }
            Console.WriteLine(sb.ToString());
            ProcessCompleteResult.AppendLine(sb.ToString());
        }
        static void S_ProcessAllDoneCallback()
        {
            //This ProcessAllDoneCallback event handler method consolidates, analyzes and displays
            //the 'kernel' function results accumulated by each ProcessCompleteCallback event
            Console.WriteLine();
            Console.WriteLine("S_ProcessAllDoneCallback():");
            Console.WriteLine("ProcessCompleteResult:");
            Console.WriteLine(ProcessCompleteResult.ToString());
            //^ Display the results of all kernel function executions

            Console.WriteLine("Click Enter to end this program");
            Console.ReadLine();

            S.Stop();
            //^ Always stop the APLNext Supervisor

            Environment.Exit(0);
            //^End this application now that the ProcessAllDoneCallback has fired
        }
    }
}