﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;

namespace TestTimeout
{
    class Program
    {
        static APLNext.Supervisor S = new APLNext.Supervisor();
        static StringBuilder ProcessCompleteResult = new StringBuilder();
        static void Main(string[] args)
        {
            try
            {
                string testPath = @"C:\APLNext\APLNextSupervisor\";
                string wsName = "SUPERVISORTEST.w3";
                //string FileName = Path.Combine(testPath, wsName);

                string SLog = Path.Combine(testPath, "Spvr.log");
                if (File.Exists(SLog))
                {      
                    File.Delete(SLog);
                }
                S.SetLogFileName(SLog);

                StringBuilder sbConfig = new StringBuilder();
                sbConfig.Append(@"<?xml version='1.0'?>");
                sbConfig.Append(@"<config><workspaces><workspace id='SUPERVISORTEST'>");
                sbConfig.Append(@"<minpool>1</minpool>");
                sbConfig.Append(@"<maxpool>2</maxpool>");
                sbConfig.Append(@"<timeout>15000</timeout>");
                //^ Timeout = 15 sec.
                sbConfig.Append(@"<debug>0</debug>");
                //^ Timeout applies
                sbConfig.Append(@"<visible>1</visible>");
                sbConfig.Append(@"<wslocation>" + Path.Combine(testPath, wsName) + "</wslocation>");
                sbConfig.Append(@"<wssize>1600000</wssize>");
                sbConfig.Append(@"<evlevel>2</evlevel>");
                sbConfig.Append(@"<busyid />");
                sbConfig.Append(@"<user_state>started</user_state>");
                sbConfig.Append(@"</workspace></workspaces></config>");
                S.OpenConfigFromText(sbConfig.ToString());
                
                S.Start();
                
                S.ProcessCompleteCallback += new APLNext.ProcessCompleteHandler(S_ProcessCompleteCallback);
                
                Int32 processId = 1;
                bool beginCallResult = S.BeginCall(1, "SUPERVISORTEST", "KernelDyadic", "rarg=" + processId.ToString(), "STOP FUNCTION");
                //^ APL+Win 'kernel' function will stop
                if (!beginCallResult)
                {
                    Console.WriteLine("BeginCall failed: processId: " + processId.ToString());
                }
                else
                {
                    Console.WriteLine("BeginCall OK: id: " + processId.ToString());
                }
                
                S.ProcessAllDoneCallback += new APLNext.ProcessAllDoneHandler(S_ProcessAllDoneCallback);
                System.Diagnostics.Process.GetCurrentProcess().WaitForExit();
                //^Wait until the ProcessAllDoneCallback event has fired
            }
            catch (Exception e)
            {
                Console.WriteLine("Error in Main(): " + e.Message);
                S.Stop();
                Console.ReadLine();
            }
        }
        static void S_ProcessAllDoneCallback()
        {
            Console.WriteLine();
            Console.WriteLine("S_ProcessAllDoneCallback():");
            Console.WriteLine("ProcessCompleteResult:");
            Console.WriteLine(ProcessCompleteResult.ToString());
            //^ Display the results of all kernel function executions
            Console.WriteLine("Click Enter to end this program");
            Console.ReadLine();
            S.Stop();
            Environment.Exit(0);
            //^End this application now that the ProcessAllDoneCallback has fired
        }

        static void S_ProcessCompleteCallback(int id, int errcode, object resultData)
        {
            StringBuilder sb = new StringBuilder();
            sb.Append("ProcessComplete: id: " + id.ToString() + " errcode: " + errcode.ToString());
            if (0 != errcode)
            {
                sb.Append(" errDesc(in resultData):");
                sb.AppendLine(resultData.ToString());
            }
            else
            {
                if (resultData == null)
                {
                    sb.Append(" resultData: null");
                }
                else
                {
                    sb.Append(" resultData: " + resultData.ToString());
                }
            }
            Console.WriteLine(sb.ToString());
            ProcessCompleteResult.AppendLine(sb.ToString());
        }
    }
}
